//===========================================================================
//  @(#) $Name: arts++-1-1-a8 $
//  @(#) $Id: ArtsBgp4RouteTable.hh,v 1.7 2000/10/02 17:05:15 dwm Exp $
//===========================================================================
//  Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU Lesser General Public
//  License, Version 2.1, February 1999, which is incorporated by
//  reference herein.
//
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual
//  property rights.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with arts++.  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//  Free Software Foundation, Inc.
//  59 Temple Place, Suite 330
//  Boston, MA 02111-1307
//  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _ARTSBGP4ROUTETABLE_HH_
#define _ARTSBGP4ROUTETABLE_HH_

#include "Arts.hh"

//----------------------------------------------------------------------------
//                  class ArtsBgp4RouteTable : public Arts                 
//----------------------------------------------------------------------------
//!  This class abstracts a BGP4 route table, but for selected routes
//!  only (1 entry per IPv4 network prefix).
//----------------------------------------------------------------------------
class ArtsBgp4RouteTable : public Arts
{
public:
  //--------------------------------------------------------------------------
  //                          ArtsBgp4RouteTable()
  //..........................................................................
  //!  constructor
  //--------------------------------------------------------------------------
  ArtsBgp4RouteTable();
  
  //--------------------------------------------------------------------------
  //                          ~ArtsBgp4RouteTable()
  //..........................................................................
  //!  destructor
  //--------------------------------------------------------------------------
  ~ArtsBgp4RouteTable();

  //--------------------------------------------------------------------------
  //  friend istream & operator >> (istream & is, 
  //                                ArtsBgp4RouteTable & artsBgp4RouteTable) 
  //..........................................................................
  //!  Reads an ArtsBgp4RouteTable from an istream.  Note this is included
  //!  for use with an istream_iterator and is not intended to be used
  //!  in any other manner; it skips over other types of arts objects
  //!  in the input stream.
  //--------------------------------------------------------------------------
  friend istream & operator >> (istream & is,
                                ArtsBgp4RouteTable & artsBgp4RouteTable);
  
};

#endif  // _ARTSBGP4ROUTETABLE_HH_
