//===========================================================================
//  Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU Lesser General Public
//  License, Version 2.1, February 1999, which is incorporated by
//  reference herein.
//
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual
//  property rights.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with arts++.  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//  Free Software Foundation, Inc.
//  59 Temple Place, Suite 330
//  Boston, MA 02111-1307
//  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

extern "C" {
  #include <sys/time.h>
  #include "caida_t.h"
  }

#include "Arts.hh"

//----------------------------------------------------------------------------
//                class ArtsRttTimeSeriesTable : public Arts 
//----------------------------------------------------------------------------
//!  This class abstracts round trip time and packet loss data.
//----------------------------------------------------------------------------
class ArtsRttTimeSeriesTable : public Arts
{
public:
  //--------------------------------------------------------------------------
  //                        ArtsRttTimeSeriesTable() 
  //..........................................................................
  //!  constructor
  //--------------------------------------------------------------------------
  ArtsRttTimeSeriesTable();

  //--------------------------------------------------------------------------
  //                        ~ArtsRttTimeSeriesTable() 
  //..........................................................................
  //!  destructor
  //--------------------------------------------------------------------------
  ~ArtsRttTimeSeriesTable();

  //--------------------------------------------------------------------------
  //     void AddRttEntry(const ArtsRttTimeSeriesTableEntry & rttEntry) 
  //..........................................................................
  //!  Adds an RTT sample to tne end of the series.
  //--------------------------------------------------------------------------
  void AddRttEntry(const ArtsRttTimeSeriesTableEntry & rttEntry);

  //--------------------------------------------------------------------------
  //      friend istream & 
  //      operator >> (istream & is, 
  //                   ArtsRttTimeSeriesTable & artsRttTimeSeriesTable) 
  //..........................................................................
  //!  This permits using istream_iterator<ArtsRttTimeSeriesTable> to
  //!  iterate over ArtsRttTimeSeriesTable objects in an istream.  We skip
  //!  over all Arts objects except ArtsRttTimeSeriesTable objects.
  //--------------------------------------------------------------------------
  friend istream &
  operator >> (istream & is,
               ArtsRttTimeSeriesTable & artsRttTimeSeriesTable);
  
};
