/* classes/include/artslocal.h.  Generated automatically by configure.  */
//===========================================================================
//  $Name: arts++-1-1-a8 $
//  $Id: artslocal.h.in,v 1.6 2000/10/02 17:05:21 dwm Exp $
//===========================================================================
//  Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU Lesser General Public
//  License, Version 2.1, February 1999, which is incorporated by
//  reference herein.
//
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual
//  property rights.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with arts++.  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//  Free Software Foundation, Inc.
//  59 Temple Place, Suite 330
//  Boston, MA 02111-1307
//  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

//---------------------------------------------------------------------------
//  The xdr_destroy() macro in <rpc/xdr.h> on FreeBSD 2.2.x systems (and
//  possibly others) is broken... it passes the XDR pointer to the
//  x_destroy() function but the x_destroy() function prototype
//  doesn't have any arguments.  C++ compilers (and rigorous C compilers)
//  don't like this, so we'll rewrite the macro to match the x_destroy()
//  prototype where we need it (in classes/src/ArtsPrimitive.cc) if
//  XDR_DESTROY_MACRO_DONT_PASS_POINTER is defined.
//---------------------------------------------------------------------------
/* #undef XDR_DESTROY_MACRO_DONT_PASS_POINTER */

//---------------------------------------------------------------------------
//  Newer versions of g++ and egcs support the Standard C++ Library
//  header file names (from ISO/IEC 14882:1998).  Unfortunately, older
//  version do not and there's a lot of legacy code written to the old
//  header names.  I'm trying to migrate toward using the standard names
//  when they exist and fall back to old g++ conventions if I can't find
//  the header under the standard name.  Hence I have a bunch of macros
//  that are set if the standard header exists under the standard name
//  and not set if the standard header is not under the standard name.
//
//  As an aside, this is one of the reasons I recommend using g++ 2.95.1
//  (there are other reasons for doing so too, mostly related to STL and
//  general template support).
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
//  If <fstream> exists, HAVE_FSTREAM should be non-zero.  Else 
//  HAVE_FSTREAM should be zero and our source will use <fstream.h>
//---------------------------------------------------------------------------
#define HAVE_FSTREAM 1

//---------------------------------------------------------------------------
//  If <iostream> exists, HAVE_IOSTREAM should be non-zero.  Else
//  HAVE_IOSTREAM should be zero and our source will use <iostream.h>
//---------------------------------------------------------------------------
#define HAVE_IOSTREAM 1

//---------------------------------------------------------------------------
//  If <iomanip> exists, HAVE_IOMANIP should be non-zero.  Else
//  HAVE_IOMANIP should be zero and our source will use <iomanip.h>
//---------------------------------------------------------------------------
#define HAVE_IOMANIP 1

//---------------------------------------------------------------------------
//  If <strstream> exists, HAVE_STRSTREAM should be non-zero.  Else
//  HAVE_STRSTREAM should be zero and our source will use <strstream.h>
//---------------------------------------------------------------------------
#define HAVE_STRSTREAM 1


