//===========================================================================
//  $Name: arts++-1-1-a8 $
//  $Id: ArtsIfIndexSelectionSet.hh,v 1.4 2000/10/02 17:05:16 dwm Exp $
//===========================================================================
//  Copyright Notice
//
//  By accessing this software, arts++, you are duly informed
//  of and agree to be bound by the conditions described below in this
//  notice:
//
//  This software product, arts++, is developed by Daniel W. McRobb, and
//  copyrighted(C) 1998 by the University of California, San Diego
//  (UCSD), with all rights reserved.  UCSD administers the CAIDA grant,
//  NCR-9711092, under which part of this code was developed.
//
//  There is no charge for arts++ software.  You can redistribute it
//  and/or modify it under the terms of the GNU Lesser General Public
//  License, Version 2.1, February 1999, which is incorporated by
//  reference herein.
//
//  arts++ is distributed WITHOUT ANY WARRANTY, IMPLIED OR EXPRESS, OF
//  MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE or that the use
//  of it will not infringe on any third party's intellectual
//  property rights.
//
//  You should have received a copy of the GNU Lesser General Public
//  License along with arts++.  Copies can also be obtained from:
//
//    http://www.gnu.org/copyleft/lesser.html
//
//  or by writing to:
//
//  Free Software Foundation, Inc.
//  59 Temple Place, Suite 330
//  Boston, MA 02111-1307
//  USA
//
//  Or contact:
//
//    info@caida.org
//===========================================================================

#ifndef _ARTSIFINDEXSELECTIONSET_HH_
#define _ARTSIFINDEXSELECTIONSET_HH_

#include "ArtsSelectionSet.hh"

//----------------------------------------------------------------------------
//    class ArtsIfIndexSelectionSet : public ArtsSelectionSet<uint16_t> 
//----------------------------------------------------------------------------
//!  This class is just an instantiation of ArtsSelectionSet<uint16_t>
//!  with a single added member for loading from a string expression
//!  of interface indexes.
//----------------------------------------------------------------------------
class ArtsIfIndexSelectionSet : public ArtsSelectionSet<uint16_t>
{
public:
  //--------------------------------------------------------------------------
  //       void Load(const char *ifIndexSelectionExpression) 
  //..........................................................................
  //!  Loads from a string expression containing a list of comma-separated
  //!  ifIndex values and hyphenated ranges of ifIndex values (inclusive).
  //--------------------------------------------------------------------------
  void Load(const char *ifIndexSelectionExpression);
};

#endif  // _ARTSIFINDEXSELECTIONSET_HH_
